//
//  IAPManager.m
//  IAPDemo
//
//  Created by Charles.Yao on 2016/10/31.
//  Copyright © 2016年 com.pico. All rights reserved.
//

#import "IAPManager.h"
#import "HSUserInfoModel.h"
#import "HSCommonTool.h"
#import "UIDevice+DeviceIDByKeychainThisDeviceOnly.h"
#import "Qonversion.h"

static NSString * const receiptKey = @"receipt_key";
static NSString * const dateKey = @"date_key";
static NSString * const userIdKey = @"userId_key";

dispatch_queue_t iap_queue() {
    static dispatch_queue_t as_iap_queue;
    static dispatch_once_t onceToken_iap_queue;
    dispatch_once(&onceToken_iap_queue, ^{
        as_iap_queue = dispatch_queue_create("com.iap.queue", DISPATCH_QUEUE_CONCURRENT);
    });
    
    return as_iap_queue;
}

@interface IAPManager ()<SKPaymentTransactionObserver, SKProductsRequestDelegate, QNPurchasesDelegate, SKPaymentQueueDelegate>

@property (nonatomic, assign) BOOL goodsRequestFinished; //判断一次请求是否完成

@property (nonatomic, copy) NSString *receipt; //交易成功后拿到的一个64编码字符串

@property (nonatomic, copy) NSString *date; //交易时间

@property (nonatomic, copy) NSString *userId; //交易人

@property (nonatomic, copy) void (^restoreFinish) (bool active, NSError* err);

@end

@implementation IAPManager

singleton_implementation(IAPManager)


#pragma mark SKProductsRequestDelegate 查询成功后的回调

- (void)productsRequest:(SKProductsRequest *)request didReceiveResponse:(SKProductsResponse *)response {
    [SVProgressHUD show];
    NSLog(@"main thread:%@",[NSThread currentThread]);
    dispatch_async(dispatch_get_main_queue(), ^{
        [SVProgressHUD show];
    });
    NSArray *product = response.products;
    
    if (product.count == 0) {
//        [HSCommandTool trackinfoUploadEventName:@"applePay_requestP_faile" pageName:@"" other:nil];

        [SVProgressHUD dismiss];
        NSLog(@"无法获取商品信息，请重试");
        
        [self filedWithErrorCode:IAP_FILEDCOED_CANNOTGETINFORMATION  appleErrorCode:-1  domain:@"null" transId:@"-1" error:@"获取商品信息为空" qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
        
        self.goodsRequestFinished = YES; //失败，请求完成

    } else {
//        [HSCommandTool trackinfoUploadEventName:@"applePay_requestP_success" pageName:@"" other:nil];
        //发起购买请求
//        SKPayment *payment = [SKPayment paymentWithProduct:product[0]];
//        payment.applicationUsername = uuid;
        SKMutablePayment *payment = [SKMutablePayment paymentWithProduct:product[0]];// 可变的
        payment.applicationUsername = [HSUserInfoModel shareUserInfo].loginModel.mid;// 发起支付时候指定用户mid
        [[SKPaymentQueue defaultQueue] addPayment:payment];
    }
}

//查询失败后的回调
- (void)request:(SKRequest *)request didFailWithError:(NSError *)error {
//    [HSCommandTool trackinfoUploadEventName:@"applePay_requestP_faile" pageName:_origin other:nil];
    [SVProgressHUD dismiss];
    
    self.goodsRequestFinished = YES; //失败，请求完成
    [self filedWithErrorCode:IAP_FILEDCOED_APPLECODE appleErrorCode:error.code  domain:error.domain transId:@"-1" error:[error localizedDescription] qonversionError:@"null"  errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
}

#pragma mark - SKPaymentQueueDelegate
// Sent when a user has initiated a purchase of a Promoted IAP from the App Store
- (BOOL)paymentQueue:(SKPaymentQueue *)paymentQueue shouldContinueTransaction:(SKPaymentTransaction *)transaction inStorefront:(SKStorefront *)newStorefront  API_AVAILABLE(ios(13.0)){
    [HSCommonTool trackinfoUploadEventName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] pageName:@"" other:@{}];
    return YES;
}

// Sent if there is a pending price consent confirmation from the App Store for the current user. Return YES to immediately show the price consent UI. Return NO if you intend to show it at a later time. Defaults to YES.
// This may be called at any time that you have transaction observers on the payment queue, so make sure to set the delegate before adding any transaction observers if you intend to implement this method.
- (BOOL)paymentQueueShouldShowPriceConsent:(SKPaymentQueue *)paymentQueue {
    [HSCommonTool trackinfoUploadEventName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] pageName:@"" other:@{}];
    return YES;
}


#pragma mark - SKPaymentTransactionObserver
// Sent when transactions are removed from the queue (via finishTransaction:).
- (void)paymentQueue:(SKPaymentQueue *)queue removedTransactions:(NSArray<SKPaymentTransaction *> *)transactions {
    [HSCommonTool trackinfoUploadEventName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] pageName:@"" other:@{}];
}

// Sent when an error is encountered while adding transactions from the user's purchase history back to the queue.
- (void)paymentQueue:(SKPaymentQueue *)queue restoreCompletedTransactionsFailedWithError:(NSError *)error {
    dispatch_async(dispatch_get_main_queue(), ^{
        [SVProgressHUD dismiss];
        if(self.restoreFinish) {
            self.restoreFinish(NO, error);
        }
    });
    [self filedWithErrorCode:IAP_FILEDCOED_CANNOTGETINFORMATION  appleErrorCode:error.code  domain:error.domain transId:@"" error:error.localizedDescription qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
}
// Sent when all transactions from the user's purchase history have successfully been added back to the queue.
- (void)paymentQueueRestoreCompletedTransactionsFinished:(SKPaymentQueue *)queue {
    bool isVip = [HSUserInfoModel shareUserInfo].loginModel.isVip;
    dispatch_async(dispatch_get_main_queue(), ^{
        [SVProgressHUD dismiss];
        if(self.restoreFinish) {
            self.restoreFinish(isVip, nil);
        }
    });

}
// Sent when the download state has changed.
- (void)paymentQueue:(SKPaymentQueue *)queue updatedDownloads:(NSArray<SKDownload *> *)downloads API_AVAILABLE(ios(6.0)) {
    [HSCommonTool trackinfoUploadEventName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] pageName:@"" other:@{}];
}

// Sent when a user initiates an IAP buy from the App Store
- (BOOL)paymentQueue:(SKPaymentQueue *)queue shouldAddStorePayment:(SKPayment *)payment forProduct:(SKProduct *)product API_AVAILABLE(ios(11.0)) {
    [HSCommonTool trackinfoUploadEventName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] pageName:@"" other:@{}];
    return YES;
}

- (void)paymentQueueDidChangeStorefront:(SKPaymentQueue *)queue API_AVAILABLE(ios(13.0)) {
    [HSCommonTool trackinfoUploadEventName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] pageName:@"" other:@{}];
}

// Sent when entitlements for a user have changed and access to the specified IAPs has been revoked.
- (void)paymentQueue:(SKPaymentQueue *)queue didRevokeEntitlementsForProductIdentifiers:(NSArray<NSString *> *)productIdentifiers API_AVAILABLE(ios(14.0)) {
    
}
// Sent when the transaction array has changed (additions or state changes).  Client should check state of transactions and finish as appropriate.
//购买操作后的回调
- (void)paymentQueue:(SKPaymentQueue *)queue updatedTransactions:(nonnull NSArray<SKPaymentTransaction *> *)transactions {
    dispatch_async(iap_queue(), ^{
        dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);
        for (SKPaymentTransaction *transaction in transactions) {
            
            switch (transaction.transactionState) {
               
                case SKPaymentTransactionStatePurchasing://正在交易
                    NSLog(@"main thread222222222:%@",[NSThread currentThread]);

                    break;

                case SKPaymentTransactionStatePurchased://交易完成
                    //提前跳转
    //                [self suceessBlockEventWithTransId:transaction.transactionIdentifier];
                    
                    [self getReceipt]; //获取交易成功后的购买凭证
                   
    //                [self saveReceipt]; //存储交易凭证
                   
                    [self verifyOrderWithReceipt:self.receipt productId: transaction.payment.productIdentifier semaphore:semaphore transaction:transaction];//把self.receipt发送到服务器验证是否有效
                    dispatch_semaphore_wait(semaphore, DISPATCH_TIME_FOREVER);
                    break;

                case SKPaymentTransactionStateFailed://交易失败
                    [self failedTransaction:transaction];
                    
                    break;

                case SKPaymentTransactionStateRestored://恢复购买
//                    [SVProgressHUD dismiss];
//
//                    [self restoreTransaction:transaction];
                    break;
               
                default:
                   
                    break;
            }
        }
    });
}
#pragma mark - QNPurchasesDelegate
- (void)qonversionDidReceiveUpdatedPermissions:(NSDictionary<NSString *,QNPermission *> * _Nonnull)permissions {
    QNPermission *premiumPermission = permissions[@"premium"];
    if (premiumPermission) {
        [HSCommonTool trackinfoUploadEventName:@"qonversionDidReceiveUpdatedPermissions" pageName:@"" other:@{@"active":@(premiumPermission.isActive),@"renew_state":@(premiumPermission.renewState),@"product_id":premiumPermission.productID}];
//        [HSUserInfoModel shareUserInfo].loginModel.isVip = premiumPermission.isActive;
    }
}
-(void)qonversionPurchaseFailWithError:(NSError *)error transaction:(SKPaymentTransaction*)transaction{
//    [self filedWithErrorCode:-1 appleErrorCode:transaction.error.code domain:transaction.error.domain transId:transaction.transactionIdentifier error:transaction.error.localizedDescription qonversionError:error.localizedDescription];
}
-(void)qonversionPurchaseSuccessWithReceipt:(NSString *)receipt transaction:(SKPaymentTransaction*)transaction{
//    dispatch_async(iap_queue(), ^{
//        dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);
//        [self verifyOrderWithReceipt:receipt transaction:transaction semaphore:semaphore];
//    });
//    [self suceessBlockEventWithTransId:transaction.transactionIdentifier];
}
# pragma mark - other
- (void)startManager { //开启监听

    dispatch_async(iap_queue(), ^{
       
        self.goodsRequestFinished = YES;
        
        /***
         内购支付两个阶段：
         1.app直接向苹果服务器请求商品，支付阶段；
         2.苹果服务器返回凭证，app向公司服务器发送验证，公司再向苹果服务器验证阶段；
         */
        
        /**
         阶段一正在进中,app退出。
         在程序启动时，设置监听，监听是否有未完成订单，有的话恢复订单。
         */
        if (@available(iOS 13.0, *)) {
            [SKPaymentQueue defaultQueue].delegate = self;
        }
        
        [[SKPaymentQueue defaultQueue] addTransactionObserver:self];
        
        /**
         阶段二正在进行中,app退出。
         在程序启动时，检测本地是否有receipt文件，有的话，去二次验证。
         */
//        [self checkIAPFiles];
        [Qonversion setPurchasesDelegate:self];
    });
}

- (void)stopManager{

    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [[SKPaymentQueue defaultQueue] removeTransactionObserver:self];
    });
}

//查询
- (void)requestProductWithId:(NSString *)productId origin:(NSString *)origin{
    _productId = productId;
    _origin = origin;
    [SVProgressHUD show];
//    [Qonversion purchase:productId completion:^(NSDictionary<NSString *,QNPermission *> * _Nonnull result, NSError * _Nullable error, BOOL cancelled, NSString * _Nullable receipt) {
//        [SVProgressHUD dismiss];
//        NSLog(@"qonversion purchase result:%@ error:%@ cancelled:%d",result,error,cancelled);
//        if (error) {
//            [self filedWithErrorCode:-1 appleErrorCode:error.code domain:error.domain transId:@"" error:error.localizedDescription qonversionError:error.localizedDescription];
//        } else {
//            QNPermission *permission = [[result allValues]firstObject];
//            dispatch_async(iap_queue(), ^{
//                dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);
//                [self verifyOrderWithReceipt:receipt productId:permission.productID semaphore:semaphore];
//            });
//            [self suceessBlockEventWithTransId:@""];
//        }
//    }];
    
    if (self.goodsRequestFinished) {
        [SVProgressHUD show];
        if ([SKPaymentQueue canMakePayments]) { //用户允许app内购

            if (productId.length) {


                NSLog(@"%@商品正在请求中",productId);

                self.goodsRequestFinished = NO; //正在请求

                NSArray *product = [[NSArray alloc] initWithObjects:productId, nil];

                NSSet *set = [NSSet setWithArray:product];

                SKProductsRequest *productRequest = [[SKProductsRequest alloc] initWithProductIdentifiers:set];

                productRequest.delegate = self;

                [productRequest start];


                NSLog(@"第一步-------");
                //请求商品
//                [HSCommandTool trackinfoUploadEventName:@"applePay_requestProduct" pageName:@"" other:nil];

            } else {

                NSLog(@"商品为空");
//                [SVProgressHUD dismiss];

                [self filedWithErrorCode:IAP_FILEDCOED_EMPTYGOODS appleErrorCode:-1 domain:@"null" transId:@"-1" error:@"获取商品为空" qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];

                self.goodsRequestFinished = YES; //完成请求
                //商品为空
//                [HSCommandTool trackinfoUploadEventName:@"applePay_productEmpty" pageName:@"" other:nil];
            }

        } else { //没有权限
//            [SVProgressHUD dismiss];

            [self filedWithErrorCode:IAP_FILEDCOED_NORIGHT appleErrorCode:-1 domain:@"null" transId:@"-1" error:@"没有权限" qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];

            self.goodsRequestFinished = YES; //完成请求
            //不允许购买
//            [HSCommandTool trackinfoUploadEventName:@"applePay_NOPermitBuy" pageName:@"" other:nil];
        }

    } else {
//        [SVProgressHUD dismiss];

        //等待上次请求
        [HSCommonTool trackinfoUploadEventName:@"applePay_WaitingLastTime" pageName:@"" other:nil];
//
        NSLog(@"上次请求还未完成，请稍等");
    }
}

-(void)suceessBlockEventWithTransId:(NSString*)tid{
    PayVerifyModel *model = [[PayVerifyModel alloc]init];
    model.is_vip = YES;
    [HSUserInfoModel shareUserInfo].loginModel.isVip = YES;

    if (self.delegate && [self.delegate respondsToSelector:@selector(verifySuccessWithModel:transId:)]) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.delegate verifySuccessWithModel:model transId:tid];
        });
   }
}

- (void)completeTransaction:(SKPaymentTransaction *)transaction {
    
    self.goodsRequestFinished = YES; //成功，请求完成
    
    [[SKPaymentQueue defaultQueue] finishTransaction: transaction];
}

- (void)failedTransaction:(SKPaymentTransaction *)transaction {

    NSLog(@"transaction.error.code = %ld", transaction.error.code);
    if (@available(iOS 12.2, *)) {
        if(transaction.error.code != SKErrorPaymentCancelled && transaction.error.code != SKErrorOverlayCancelled) {
            [self filedWithErrorCode:IAP_FILEDCOED_BUYFILED appleErrorCode:transaction.error.code domain:transaction.error.domain transId:transaction.transactionIdentifier error:transaction.error.localizedDescription qonversionError:@"null"  errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
        } else {
            
            [self filedWithErrorCode:IAP_FILEDCOED_USERCANCEL appleErrorCode:transaction.error.code domain:transaction.error.domain transId:transaction.transactionIdentifier error:transaction.error.localizedDescription qonversionError:@"null"  errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
        }
    } else {
        if(transaction.error.code != SKErrorPaymentCancelled) {
            [self filedWithErrorCode:IAP_FILEDCOED_BUYFILED appleErrorCode:transaction.error.code domain:transaction.error.domain transId:transaction.transactionIdentifier error:transaction.error.localizedDescription qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
        } else {

            [self filedWithErrorCode:IAP_FILEDCOED_USERCANCEL appleErrorCode:transaction.error.code domain:transaction.error.domain transId:transaction.transactionIdentifier error:transaction.error.localizedDescription qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
        }
    }

    [[SKPaymentQueue defaultQueue] finishTransaction: transaction];
   
    self.goodsRequestFinished = YES; //失败，请求完成

}


- (void)restoreWithFinish:(void (^)(bool,NSError*))finish {
    self.restoreFinish = finish;
    [SVProgressHUD showWithStatus:@"Loading..."];
//    [Qonversion restoreWithCompletion:^(NSDictionary<NSString *,QNPermission *> * _Nonnull result, NSError * _Nullable error) {
//      if (error) {
//        // Handle error
//          NSLog(@"%@",error);
//          finish(false);
//      }
//      QNPermission *permission = result[@"active"];
//      if (permission && permission.isActive) {
//        // Restored and permission is active
//          [HSUserInfoModel shareUserInfo].loginModel.isVip = YES;
//          finish(true);
//      } else {
//          finish(false);
//      }
//        self.goodsRequestFinished = YES; //恢复购买，请求完成
//        [SVProgressHUD dismiss];
//    }];
//    [[SKPaymentQueue defaultQueue] finishTransaction: transaction];
    [[SKPaymentQueue defaultQueue] restoreCompletedTransactions];
}
//获取交易成功后的购买凭证
- (void)getReceipt {

    NSURL *receiptUrl = [[NSBundle mainBundle] appStoreReceiptURL];

    NSData *receiptData = [NSData dataWithContentsOfURL:receiptUrl];

    self.receipt = [receiptData base64EncodedStringWithOptions:0];
//    if (self.receipt.length > 0) {
//        [HSCommandTool trackinfoUploadEventName:@"applePay_paySuccess_getReceipt_success" pageName:@"" other:nil];
//    }else{
//        [HSCommandTool trackinfoUploadEventName:@"applePay_paySuccess_getReceipt_fail" pageName:@"" other:nil];
//    }
}

//持久化存储用户购买凭证(这里最好还要存储当前日期，用户id等信息，用于区分不同的凭证)
//-(void)saveReceipt {
//
//    self.date = [NSDate chindDateFormate:[NSDate date]];
//
//    NSString *fileName = [NSString uuid];
//
//    self.userId = @"UserID";
//
//    NSString *savedPath = [NSString stringWithFormat:@"%@/%@.plist", [SandBoxHelper iapReceiptPath], fileName];
//
//    NSDictionary *dic =[NSDictionary dictionaryWithObjectsAndKeys:
//                        self.receipt,                           receiptKey,
//                        self.date,                              dateKey,
//                        self.userId,                            userIdKey,
//                        nil];
//
//    NSLog(@"%@",savedPath);
//
//    [dic writeToFile:savedPath atomically:YES];
//    //存储凭证
//    [HSCommandTool trackinfoUploadEventName:@"applePay_paySuccess_saveReceipt" pageName:@"" other:nil];
//}

//将存储到本地的IAP文件发送给服务端 验证receipt失败,App启动后再次验证
//- (void)checkIAPFiles{
//
//    NSFileManager *fileManager = [NSFileManager defaultManager];
//
//    NSError *error = nil;
//
//    //搜索该目录下的所有文件和目录
//    NSArray *cacheFileNameArray = [fileManager contentsOfDirectoryAtPath:[SandBoxHelper iapReceiptPath] error:&error];
//
//    if (error == nil) {
//       //使用信号量控制发送
//        dispatch_async(dispatch_get_global_queue(0, 0), ^{  //异步
//            [HSCommandTool trackinfoUploadEventName:@"applePay_takeReceipt_success" pageName:@"" other:nil];
//
//               dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);
//
//                  for (NSString *name in cacheFileNameArray) {
//                      if ([name hasSuffix:@".plist"]){ //如果有plist后缀的文件，说明就是存储的购买凭证
//                          NSString *filePath = [NSString stringWithFormat:@"%@/%@", [SandBoxHelper iapReceiptPath], name];
//
//                          NSDictionary *dic = [NSDictionary dictionaryWithContentsOfFile:filePath];
//
//                          //这里的参数请根据自己公司后台服务器接口定制，但是必须发送的是持久化保存购买凭证
//                          NSMutableDictionary *params = [NSMutableDictionary dictionaryWithObjectsAndKeys:
//                                                         [dic objectForKey:receiptKey],          receiptKey,
//                                                         [dic objectForKey:dateKey],             dateKey,
//                                                         [dic objectForKey:userIdKey],           userIdKey,
//                                                         nil];
//                          NSString *receiptStr = [dic objectForKey:receiptKey];
//                          if(receiptStr.length > 0){
//                              [self verifyOrderWithReceipt:receiptStr semaphore:semaphore];
////                              [self removeReceipt];
//
//                          } else {//凭证无效
//
//                              //做你想做的
//                          }
//                      }
//                  }
//           });
//
//    } else {
//        [HSCommandTool trackinfoUploadEventName:@"applePay_takeReceipt_faile" pageName:@"" other:nil];
//        NSLog(@"AppStoreInfoLocalFilePath error:%@", [error domain]);
//    }
//}


//将凭证发送给后台
-(void)verifyOrderWithReceipt:(NSString *)receipt productId:(NSString*)productId semaphore:(NSObject<OS_dispatch_semaphore> *)semaphore transaction:(SKPaymentTransaction*)transaction{
    if (receipt == nil || receipt.length == 0) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_global_queue(0, 0), ^{
            dispatch_semaphore_signal(semaphore);
        });
        [self filedWithErrorCode:IAP_FILEDCODE_VERIFY appleErrorCode:-1 domain:@"null" transId:@"" error:@"订单为空" qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
        return;
    }
    
    WEAKSELF
    NSInteger mid = transaction.payment.applicationUsername ? transaction.payment.applicationUsername.integerValue : [HSUserInfoModel shareUserInfo].loginModel.mid.integerValue;
    NSDictionary *param = @{@"receipt_data":receipt,@"product_id":productId,@"origin":weakSelf.origin.length > 0 ? weakSelf.origin : @"",@"mid":@(mid)};
    
    [AQHTTPRequest postApplePayOrderWithParameters:param success:^(id  _Nonnull response) {
        STRONGSELF
        NSLog(@"交易验证后的返回数据是%@",response);
        //信号量放行
        
        //移除凭证
//        [self removeReceipt];
        
        PayVerifyModel *model = [PayVerifyModel mj_objectWithKeyValues:response];
        if (model.is_vip) {
            [strongSelf suceessBlockEventWithTransId:transaction.transactionIdentifier];
            NSLog(@"同步了vip状态");
        } else {
            [strongSelf filedWithErrorCode:IAP_FILEDCODE_VERIFY appleErrorCode:-1 domain:@"null" transId:transaction.transactionIdentifier error:@"获取用户状态不是vip" qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
                [strongSelf suceessBlockEventWithTransId:transaction.transactionIdentifier];
        }
        [strongSelf completeTransaction:transaction];
        
        
        dispatch_semaphore_signal(semaphore);
//        PayVerifyModel *model = [PayVerifyModel mj_objectWithKeyValues:response];
//
//        [HSUserInfoModel shareUserInfo].loginModel.isVip = YES;
//
//            if (weakSelf.delegate && [weakSelf.delegate respondsToSelector:@selector(verifySuccessWithModel:)]) {
//                [weakSelf.delegate verifySuccessWithModel:model];
//            }
    } failure:^(NSError * _Nonnull error) {
        //移除凭证
//        [self removeReceipt];
        [self filedWithErrorCode:IAP_FILEDCODE_VERIFY appleErrorCode:error.code  domain:error.domain transId:@"" error:@"验证订单失败" qonversionError:@"null" errFuncName:[NSString stringWithCString:__FUNCTION__ encoding:NSUTF8StringEncoding] errFileName:[NSString stringWithCString:__FILE__ encoding:NSUTF8StringEncoding] errLineNum:__LINE__];
        
        //信号量放行
        dispatch_semaphore_signal(semaphore);
    }];
}

//验证成功就从plist中移除凭证
//-(void)removeReceipt{
//
//    NSFileManager *fileManager = [NSFileManager defaultManager];
//
//    if ([fileManager fileExistsAtPath:[SandBoxHelper iapReceiptPath]]) {
//
//        [fileManager removeItemAtPath:[SandBoxHelper iapReceiptPath] error:nil];
//
//    }
//}


//错误信息反馈
- (void)filedWithErrorCode:(NSInteger)errorCode appleErrorCode:(NSInteger)appleErrorCode domain:(NSString*)domain transId:(NSString*)tid error:(NSString *)error qonversionError:(NSString*)qError errFuncName:(NSString*)errFuncName errFileName:(NSString*)errFileName errLineNum:(int)errLineNum{
    dispatch_async(dispatch_get_main_queue(), ^{

        if (self.delegate && [self.delegate respondsToSelector:@selector(filedWithErrorCode:appleErrorCode: domain:transId:error:qonversionError:errFuncName:errFileName:errLineNum:)]) {
//            [self.delegate filedWithErrorCode:code appleErrorCode:appleErrorCode domain:domain transId:tid andError:error qonversionError:qError];
            
//            switch (code) {
//                case IAP_FILEDCOED_APPLECODE:
//
//                    break;
//
//                    case IAP_FILEDCOED_NORIGHT:
//                    [self.delegate filedWithErrorCode:IAP_FILEDCOED_NORIGHT appleErrorCode:appleErrorCode domain:domain transId:tid error:error qonversionError:qError errFuncName:errFuncName errFileName:errFileName errLineNum:errLineNum];
//                    break;
//
//                case IAP_FILEDCOED_EMPTYGOODS:
//                    [self.delegate filedWithErrorCode:IAP_FILEDCOED_EMPTYGOODS appleErrorCode:appleErrorCode domain:domain transId:tid error:error qonversionError:qError errFuncName:errFuncName errFileName:errFileName errLineNum:errLineNum];
//                    break;
//
//                case IAP_FILEDCOED_CANNOTGETINFORMATION:
//                     [self.delegate filedWithErrorCode:IAP_FILEDCOED_CANNOTGETINFORMATION appleErrorCode:appleErrorCode domain:domain transId:tid andError:error qonversionError:qError errFuncName:errFuncName errFileName:errFileName errLineNum:errLineNum];
//                    break;
//
//                case IAP_FILEDCOED_BUYFILED:
//                     [self.delegate filedWithErrorCode:IAP_FILEDCOED_BUYFILED appleErrorCode:appleErrorCode domain:domain transId:tid andError:error qonversionError:qError errFuncName:errFuncName errFileName:errFileName errLineNum:errLineNum];
//                    break;
//
//                case IAP_FILEDCOED_USERCANCEL:
//                     [self.delegate filedWithErrorCode:IAP_FILEDCOED_USERCANCEL appleErrorCode:appleErrorCode domain:domain transId:tid andError:error qonversionError:qError errFuncName:errFuncName errFileName:errFileName errLineNum:errLineNum];
//                    break;
//
//                case IAP_FILEDCODE_VERIFY:
//                     [self.delegate filedWithErrorCode:IAP_FILEDCODE_VERIFY appleErrorCode:appleErrorCode domain:domain transId:tid andError:error qonversionError:qError errFuncName:errFuncName errFileName:errFileName errLineNum:errLineNum];
//                    break;
//
//                default:
//                    break;
//            }
            [self.delegate filedWithErrorCode:errorCode appleErrorCode:appleErrorCode domain:domain transId:tid error:error qonversionError:qError errFuncName:errFuncName errFileName:errFileName errLineNum:errLineNum];
        }
    });
}

-(void)checkPermissons {
    [Qonversion checkPermissions:^(NSDictionary<NSString *,QNPermission *> * _Nonnull permissions,
                                   NSError * _Nullable error) {
      
        QNPermission *premiumPermission = permissions[@"premium"];
        if (premiumPermission) {
            [HSCommonTool trackinfoUploadEventName:@"qonversion_check_permisson" pageName:@"" other:@{@"active":@(premiumPermission.isActive),@"renew_state":@(premiumPermission.renewState),@"product_id":premiumPermission.productID}];
//            [HSUserInfoModel shareUserInfo].loginModel.isVip = premiumPermission.isActive;
        } else {
//            [HSUserInfoModel shareUserInfo].loginModel.isVip = false;
        }
    }];
}
@end
